'''
Created on 5 Jul 2021

@author: jacklok
'''
from flask_httpauth import HTTPBasicAuth
from flask import request, session, abort
import logging
from functools import wraps
from trexlib.utils.string_util import is_not_empty
from trexmodel.models.datastore.merchant_models import MerchantAcct,\
    MerchantUser


logger = logging.getLogger('debug')

class HTTPBasicAuthWrapper(HTTPBasicAuth):
    def login_required(self, f=None, role=None, optional=None):
        
        auth        = self.get_auth()
        username    = auth.username
        
        api_key     = request.headers.get('x-api-key')
        
        logger.debug('auth=%s', auth)
        logger.debug('api_key=%s', api_key)
        
        if is_not_empty(api_key):
            
            merchant_acct   = MerchantAcct.get_by_api_key(api_key)
            merchant_user   = MerchantUser.get_by_username(username) 
            if merchant_user.merchant_acct_key == merchant_acct.key_in_str:
            
                return super(self).login_required(f, role=role, optional=optional)
            else:
                abort(401)
        else:
            abort(401)



from wtforms import StringField, PasswordField, validators, DateField
from trexadmin.forms.base_forms import ValidationBaseForm
from trexadmin.libs.wtforms import validators as custom_validator
from flask_babel import gettext
from datetime import date

class UserRegistrationForm(ValidationBaseForm):
    name                = StringField(gettext('Name'), validators=[
                                        validators.InputRequired(gettext('Name is required')),
                                        validators.Length(min=3, max=300, message='Name length must be within 3 and 300 characters'),
                                        
                                        ]
                                        )
    email               = StringField('Email Address', validators=[
                                        validators.Email(gettext("Please enter valid email address.")),
                                        validators.InputRequired(gettext('Email is required')),
                                        ]
                                        )
    mobile_phone        = StringField('Mobile Phone', validators=[
                                        #validators.InputRequired(gettext('Mobile Phone is required')),
                                        ]
                                        )
    
    gender              = StringField('Gender', validators=[
                                        validators.InputRequired(gettext('Gender is required')),
                                        ]
                                        )
    
    birth_date          = StringField('Mobile Phone', validators=[
                                        validators.InputRequired(gettext('Birth date is required')),
                                        ]
                                        )
    
    password            = StringField(gettext('Password'), validators=[
                                        validators.InputRequired(gettext('Password is required')),
                                        
                                        ]
                                        )
    
class UserDetailsForm(UserRegistrationForm):
    
    gender              = StringField(gettext('Gender'), [
                                        validators.Optional(),
                                        validators.Length(min=1, max=1, message=gettext('Gender value is either m or f')),
                                        
                                        ]
                                        )
    
    birth_date          = DateField('Date of Birth', format='%d/%m/%Y', validators=[
                                            validators.Optional(),
                                        ])
    
    mobile_phone        = StringField('Mobile Phone', validators=[
                                        custom_validator.RequiredIfOtherFieldEmpty(
                                                        ['email'],
                                                        message=gettext("Either email or mobile phone is required"),
                                                        
                                                        ),
                                        
                                        ]
                                        )    


'''
Created on 28 Oct 2021

@author: jacklok
'''

from flask import Blueprint, request, url_for, jsonify
import logging, json
from trexapi.decorators.api_decorators import auth_token_required
from trexlib.utils.string_util import is_not_empty, is_empty
from trexadmin.libs.http import create_rest_message
from trexadmin.libs.http import StatusCode
from trexmodel.models.datastore.pos_models import POSSetting
from trexmodel.utils.model.model_util import create_db_client
from trexmodel.models import merchant_helpers
from trexmodel.models.datastore.product_models import ProductCatalogue,\
    ProductCategory
from trexlib.utils.common.common_util import sort_list
from firebase_admin import firestore
from datetime import datetime
from trexapi.conf import API_ERR_CODE_INVALID_ACTIVATION_CODE


#logger = logging.getLogger('api')
logger = logging.getLogger('debug')

pos_api_bp = Blueprint('pos_api_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1/pos')

@pos_api_bp.route('/check-activation', methods=['POST'])
def check_activation():
    
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    logger.debug('activation_code=%s', activation_code)
    
    if is_not_empty(activation_code):
        db_client = create_db_client(caller_info="check_activation")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
        
        if pos_setting:
            if pos_setting.activated==False:
                return create_rest_message(status_code=StatusCode.OK)
            else:
                return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
        else:
            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)

def getPOSAccountSetting(activation_code):
    if is_not_empty(activation_code):
        db_client = create_db_client(caller_info="getPOSAccountSetting")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
        
        if pos_setting:
            logger.info('Found POS setting');
            #if pos_setting.activated==False:
            if True:
                logger.info('POS activation code is valid');
                pos_setting_details = None
                with db_client.context():
                    #is_activated = POSSetting.activate(activation_code)
                    #if is_activated:
                    #merchant_acct                                       = pos_setting.merchant_acct_entity
                    outlet                                              = pos_setting.assigned_outlet_entity
                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(outlet, device_setting=pos_setting) 
                    
                    pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
                
                #if is_activated:
                if True:
                    
                    logger.debug('pos_setting_details=%s', pos_setting_details);
                    
                    return create_rest_message(status_code=StatusCode.OK,
                                               **pos_setting_details
                                               )
                else:
                    return create_rest_message('Failed to activate', status_code=StatusCode.BAD_REQUEST)
            else:
                logger.info('POS activation code have been used');
                return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
        else:
            logger.info('POS setting record is not found');
            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, error_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
    else:
        logger.info('activation_code is empty');
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
    
    
def updateActivationAndGetPOSAccountSetting(activation_code, device_id):
    if is_not_empty(activation_code) and is_not_empty(device_id):
        db_client = create_db_client(caller_info="updateActivationAndGetPOSAccountSetting")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
        
        if pos_setting:
            logger.info('POS activation code is valid');
            
            is_valid = False
            pos_setting_details = None
            
            with db_client.context():
                pos_setting = POSSetting.get_by_activation_code(activation_code)
                if pos_setting.activated:
                    if pos_setting.device_id == device_id:
                        is_valid = True
                else:
                    is_valid = True
                    pos_setting.activate(device_id)
            
                if is_valid:    
                    pos_setting_details                                 = merchant_helpers.construct_setting_by_outlet(pos_setting.assigned_outlet_entity, device_setting=pos_setting)
                    #pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
            
            if is_valid:
                
                logger.debug('pos_setting_details=%s', pos_setting_details);
                
                return create_rest_message(status_code=StatusCode.OK,
                                           **pos_setting_details
                                           )
            else:
                if pos_setting.activated and pos_setting.device_id != device_id:
                    return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
                else:
                    return create_rest_message('Failed to activate', status_code=StatusCode.BAD_REQUEST)
            
        else:
            logger.info('POS setting record is not found');
            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
    else:
        logger.info('activation_code is empty or device id is empty');
        return create_rest_message('Activation code and device id are required', status_code=StatusCode.BAD_REQUEST)
        
    
@pos_api_bp.route('/account-sync', methods=['get'])
def activate():
    try:
        activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
        logger.debug('activation_code=%s', activation_code)
        return getPOSAccountSetting(activation_code)
    except:
        logger.error('Missing activation code')
        return create_rest_message('Missing activate code', status_code=StatusCode.BAD_REQUEST)
    
@pos_api_bp.route('/activate', methods=['POST'])
def activate_post():
    
    try:
        
        activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
        device_id       = request.args.get('device_id') or request.form.get('device_id') or request.json.get('device_id')
        
        logger.debug('activation_code=%s', activation_code)
        logger.debug('device_id=%s', device_id)
        
        return updateActivationAndGetPOSAccountSetting(activation_code, device_id)
    
    except:
        logger.error('Missing activation code')
        return create_rest_message('Missing activate code', status_code=StatusCode.BAD_REQUEST)    
       
    
def parse_category_to_json(category_tree_structure):
    data_list = []
    
    for category in category_tree_structure:
        data = {
                    'code'                  : category.get('key'),
                    'label'                 : category.get('category_label'),
                    'label_with_item_count' : category.get('category_label_and_other_details'),
                    'group'                 : category.get('has_child'),
                    'product_modifier'      : category.get('product_modifier'),
                    'product_items'         : category.get('product_items'),
                }
        if category.get('childs'):
            child_data_list = parse_category_to_json(category.get('childs'))
            if child_data_list:
                data['childs'] = child_data_list   
        
        data_list.append(data)
    
    return data_list

def construct_category_tree_structure(category_tree_structure, category_list):
    for category in category_list:
        if is_empty(category.get('parent_category_key')):
            #top most category
            category_tree_structure.append(category)
            __find_child_category(category, category_list)
                

def __lookup_category_from_category_list(category_code, category_list):
    for category in category_list:
        if category.get('key') == category_code:
            return category  

    
def __find_child_category(category, category_list):
    if is_not_empty(category.get('child_category_keys')):
        childs                      = []
        parent_product_modifier     = category.get('product_modifier') or []
        
        for child_category_key in category.get('child_category_keys'):
            child = __lookup_category_from_category_list(child_category_key, category_list)
            
            logger.debug('category product_modifier of %s =%s', category.get('key'), parent_product_modifier)
            if child:
                child_product_modifier      = child.get('product_modifier') or []
                child_product_modifier      = list(set(parent_product_modifier) | set(child_product_modifier) )
                child['product_modifier']   = child_product_modifier
                
                logger.debug('child_product_modifier of %s =%s', category.get('key'), child_product_modifier)
                
                if is_not_empty(child.get('child_category_keys')):
                    __find_child_category(child, category_list)
                childs.append(child)
        
        category['childs'] = childs

def get_product_category_structure_code_label_json(merchant_acct):
    category_list       = get_product_category_structure(merchant_acct)
    
    category_tree_structure = []
    
    construct_category_tree_structure(category_tree_structure, category_list)
    
    return parse_category_to_json(category_tree_structure)

def get_product_category_structure(merchant_acct):
    
    sorted_category_structure = []
    
    category_structure      = ProductCategory.get_structure_by_merchant_acct(merchant_acct)
    category_structure      = sort_list(category_structure, sort_attr_name='category_label')
    
    #logger.debug('category_structure=%s', category_structure)
    
    for c in category_structure:
        sorted_category_structure.append(c.to_dict())
    
    
    logger.debug('sorted_category_structure=%s', sorted_category_structure)
            
    return sorted_category_structure

@pos_api_bp.route('/check-catalogue-status', methods=['GET'])
@auth_token_required
def check_catalogue_status():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    
    if is_not_empty(activation_code):
        pos_setting = None
        db_client = create_db_client(caller_info="check_catalogue_status")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
            if pos_setting:
                
                assigned_outlet = pos_setting.assigned_outlet_entity
                
                catalogue_key   = assigned_outlet.assigned_catalogue_key
                
                if is_not_empty(catalogue_key):
                    product_catalogue   = ProductCatalogue.fetch(catalogue_key)
                    if product_catalogue:
                        return {
                                    'last_updated_datetime' : product_catalogue.modified_datetime.strftime('%d-%m-%Y %H:%M:%S')
                                } 
                    else:
                        return create_rest_message('Invalid catalogue data', status_code=StatusCode.BAD_REQUEST)
        
        if pos_setting is None:
            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, err_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
        else:
            return create_rest_message('Not catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
                
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
    
@pos_api_bp.route('/get-catalogue', methods=['GET'])
#@auth_token_required
def get_catalogue():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    
    logger.debug('activation_code=%s', activation_code);
    
    if is_not_empty(activation_code):
        pos_setting         = None
        db_client           = create_db_client(caller_info="get_catalogue")
        valid_return        = None
        product_catalogue   = None
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
            if pos_setting:
                
                assigned_outlet = pos_setting.assigned_outlet_entity
                
                logger.info('assigned_outlet name=%s', assigned_outlet.name)
                
                catalogue_key   = assigned_outlet.assigned_catalogue_key
                
                logger.debug('catalogue_key=%s', catalogue_key);
                
                if is_not_empty(catalogue_key):
                    product_catalogue   = ProductCatalogue.fetch(catalogue_key)
                    merchant_acct       = pos_setting.merchant_acct_entity
                    
                    #category_tree_structure_in_json  = json.dumps(get_product_category_structure_code_label_json(merchant_acct), sort_keys = True, separators = (',', ': '))
                    category_tree_structure_in_json  = get_product_category_structure_code_label_json(merchant_acct)
                    
                    if product_catalogue:
                        last_updated_datetime = product_catalogue.modified_datetime
                        if assigned_outlet.modified_datetime is not None and assigned_outlet.modified_datetime>last_updated_datetime:
                            last_updated_datetime = assigned_outlet.modified_datetime
                        valid_return =  {
                                                'key'                       : catalogue_key,    
                                                'category_list'             : category_tree_structure_in_json,
                                                'product_by_category_map'   : product_catalogue.published_menu_settings,
                                                'last_updated_datetime'     : last_updated_datetime.strftime('%d-%m-%Y %H:%M:%S')
                                            } 
                    
                
                    
        if valid_return:
            return valid_return
        else:
            if product_catalogue==None:
                logger.debug('No catalogue have been assigned or published');
                return create_rest_message('No catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
            else:
                if pos_setting is None:
                    logger.debug('Invalid activation code');
                    return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST, err_code=API_ERR_CODE_INVALID_ACTIVATION_CODE)
                else:
                    logger.debug('No catalogue have been assigned or published');
                    return create_rest_message('No catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
                
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)    

@pos_api_bp.route('/version-sync', methods=['get'])
def version_sync():
    db_client       = create_db_client(caller_info="version_sync")
    
    with db_client.context():
        version =  {
                                '''
                                'setting':[
                                            {
                                                "table_name": "setting",
                                                "version" : 1,
                                                "script": "ALTER TABLE setting ADD COLUMN account_settings TEXT",
                                            }
                                               
                                        ],
                                'user'  :[
                                            {
                                                "table_name": "user",
                                                "version" : 1,
                                                "script": "ALTER TABLE user ADD COLUMN token_expiry_datetime TEXT",
                                                
                                            }
                                        ]
                                        
                                '''
                                'setting'  :[
                                            
                                        ]
                                       
                            }
                                
    
            
    
    return version

@pos_api_bp.route('/sales-overview/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['GET'])
#@auth_token_required
def sales_overview_stat(merchant_code, outlet_key, transact_date):
    total_sales_amount              = .0
    total_tax_amount                = .0
    total_service_charge_amount     = .0
    total_paid_count                = 0
    total_voided_count              = 0
    total_item_sold_count           = 0
    
    firebase        = firestore.client()        
    invoice_path    = 'merchant/%s/outlet/%s/date/%s/invoice' % (merchant_code, outlet_key, transact_date)
    
    logger.debug('invoice_path=%s', invoice_path)
    
    invoice_list_ref = firebase.collection(invoice_path).stream()
    #invoice_list_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(outlet_key).collection('date').document(transact_date).collection('invoice').stream()
    
    
    try:
        for invoice_ref in invoice_list_ref:
            invoice = invoice_ref.to_dict()
            logger.debug('invoice=%s', invoice)
            
            if invoice.get('status') == 'voided':
                total_voided_count+=1
                
            elif invoice.get('status') == 'paid':
                payment_details     = json.loads(invoice.get('payment_details'))
                transaction_items   = json.loads(invoice.get('transaction_items'))
                total_sales_amount+= payment_details.get('net_total_amount')
                total_paid_count+=1
                
                for tax_details in payment_details.get('tax_details_list'):
                    total_tax_amount+=tax_details.get('tax_amount')
                    
                for service_charge_details in payment_details.get('service_charge_details_list'):
                    total_service_charge_amount+=service_charge_details.get('charge_amount')
                
                for item in transaction_items:
                    total_item_sold_count+=item.get('ordered_quantity')
                
                
            
                
    except Exception as e:
        logger.error('%s', e)
    
    return jsonify({
                    'total_paid_count'              : total_paid_count,
                    'total_voided_count'            : total_voided_count,
                    'total_sales_amount'            : total_sales_amount,
                    'total_tax_amount'              : total_tax_amount,
                    'total_service_charge_amount'   : total_service_charge_amount,
                    'total_item_sold_count'         : total_item_sold_count,
                    })
    
  
#@pos_api_bp.route('/counter/<date_value>', methods=['POST'])
#@auth_token_required
@pos_api_bp.route('/counter/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['POST'])
def counter_post(merchant_code, outlet_key, transact_date):
    value = request.args.get('value') or request.form.get('value') or request.json.get('value')
    firebase = firestore.client()
           
    #counter_ref = firebase.collection('test').document('counter').collection(date_value).document('details')
    counter_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(str(outlet_key)).collection('date').document(transact_date).collection('counter').document('details')
    
    if counter_ref is None:
        logger.debug('Counter is not found')
        counter_ref.set({'value':value})
    else:    
        logger.debug('Counter is found')
        counter_ref.set({'value':value})
    
    return 'OK'

#@pos_api_bp.route('/counter/<date_value>', methods=['GET'])
@pos_api_bp.route('/counter/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['GET'])
#@auth_token_required
def counter_get(merchant_code, outlet_key, transact_date):
    firebase = firestore.client()
     
    counter_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(str(outlet_key)).collection('date').document(transact_date).collection('counter').document('details')
    #counter_ref = firebase.collection('test').document('counter').collection(date_value).document('details')
    counter_value = 0
    if counter_ref is not None:
        counter_value = counter_ref.get().get('value')
    
    return jsonify({
                    'value': counter_value,
                })



 


'''
Created on 30 Jun 2021

@author: jacklok
'''

from flask import Blueprint, request, Response, session 
from flask_restful import Resource, abort
import logging
from trexlib.utils.log_util import get_tracelog
from flask_httpauth import HTTPBasicAuth
from trexapi import conf
from flask_restful import Api
from trexmodel.utils.model.model_util import create_db_client
from trexmodel.models.datastore.merchant_models import MerchantUser,\
    MerchantAcct
import hashlib
from random import random
from trexlib.utils.string_util import random_string, is_not_empty
from flask.json import jsonify
from datetime import datetime, timedelta
from trexapi import conf as api_conf
from trexlib.utils.crypto_util import encrypt_json
from trexapi.decorators.api_decorators import auth_token_required
from trexapi.libs.flask_auth_wrapper import HTTPBasicAuthWrapper

#logger = logging.getLogger('api')
logger = logging.getLogger('debug')

auth = HTTPBasicAuth()
#auth = HTTPBasicAuthWrapper()


api_bp = Blueprint('api_base_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1')

merchant_api_bp = Blueprint('merchant_api_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1/merchant')

api = Api(api_bp)

merchant_api = Api(merchant_api_bp)



@auth.verify_password
def verify_user_auth(username, password):
    if not (username and password):
        return False
    
    db_client   = create_db_client(caller_info="verify_user_auth")
    valid_auth  = False
    
    with db_client.context():
        merchant_user = MerchantUser.get_by_username(username)
        
        if merchant_user:
            
            md5_hashed_password = hashlib.md5(password.encode('utf-8')).hexdigest()
            
            logger.debug('verify_user_auth: username=%s', username)
            logger.debug('verify_user_auth: password=%s', password)
            logger.debug('verify_user_auth: md5_hashed_password=%s', md5_hashed_password)
            
            if merchant_user.is_valid_password(md5_hashed_password):
                valid_auth = True
            else:
                logger.warn('Invalid merchant password')
        else:
            logger.warn('Invalid merchant username=%s', username)    
        
    return valid_auth

def __generate_random():
    return hashlib.md5(str(random_string(6)).encode('utf-8')).hexdigest()

def default_generate_nonce():
    session["auth_nonce"] = __generate_random()
    return session["auth_nonce"]
 
def default_verify_nonce(nonce):
    return nonce == session.get("auth_nonce")

def default_generate_opaque():
    session["auth_opaque"] = __generate_random()
    return session["auth_opaque"]

def default_verify_opaque(opaque):
    return opaque == session.get("auth_opaque")
    
class APIBaseResource(Resource):
    @property
    def realm(self):
        return 'base'
    
    def generate_ha1(self, username, password):
        a1 = username + ":" + self.realm + ":" + password
        a1 = a1.encode('utf-8')
        return hashlib.md5(a1).hexdigest()
    
    def generate_token(self, acct_id, username):
        expiry_datetime = datetime.now() + timedelta(minutes = int(api_conf.API_TOKEN_EXPIRY_LENGTH_IN_MINUTE))
        
        logger.debug('expiry_datetime=%s', expiry_datetime)
        
        token_content =  {
                            'acct_id'           : acct_id,
                            'username'          : username,
                            'expiry_datetime'   : expiry_datetime.strftime('%d-%m-%Y %H:%M:%S'),
                            }
        
        logger.debug('token_content=%s', token_content)
        
        return (expiry_datetime, encrypt_json(token_content))
    
    
class APIVersionResource(APIBaseResource):
    
    @auth.login_required
    def get(self):
        output_json =  {
                        'version'   :   conf.VERSION,
                        'username'  :   auth.current_user()
                        }
        
        return output_json
    
class APIResource(APIBaseResource):
    
    def get(self):
        return conf.VERSION    
    
class AuthenticateAPIResource(APIBaseResource):  
    
    @auth.login_required
    def post(self):
        username    = auth.current_user()
        
        acct_id = request.headers.get('x-acct-id')
        api_key = request.headers.get('x-api-key')
        
        logger.debug('acct_id=%s', acct_id)    
        logger.debug('api_key=%s', api_key)
        
        is_authenticated = False
        output_json = {}
        
        if is_not_empty(acct_id) and is_not_empty(api_key):
            db_client = create_db_client(caller_info="AuthenticateAPIResource.post")
            with db_client.context():
                merchant_acct = MerchantAcct.fetch(acct_id)
            
                if merchant_acct:
                    if api_key == merchant_acct.api_key:
                        merchant_user               = MerchantUser.get_by_username(username)
                        (expiry_datetime, token)    = self.generate_token(acct_id, username)
                        #session['auth_username']    = username
                        #session['acct_id']          = acct_id
                        
                        logger.debug('token=%s', token)
                        logger.debug('auth_username=%s', username)
            
                        output_json =  {
                                        'auth_token'        : token,
                                        'expires_in'        : int(api_conf.API_TOKEN_EXPIRY_LENGTH_IN_MINUTE) * 60,
                                        #'expiry_datetime'   : expiry_datetime.strftime('%d-%m-%Y %h:%M:$S'),
                                        'granted_outlet'    : merchant_user.granted_outlet_details_list,
                                        'username'          : merchant_user.username,
                                        'name'              : merchant_user.name,
                                        'is_admin'          : merchant_user.is_admin,
                                        'granted_access'    : merchant_user.permission.get('granted_access'),
                                        'gravatar_url'      : merchant_user.gravatar_url,
                                        }
            
                        logger.debug('output_json=%s', output_json)
                        
                        is_authenticated = True
                    
            if is_authenticated:
                return output_json
                    
        abort(401)
        
        


class SecureAPIResource(AuthenticateAPIResource):  
    
    def __init__(self):
        super(SecureAPIResource, self).__init__()  
    
    
class CheckAuthTokenResource(SecureAPIResource):
    
    @auth_token_required
    def get(self):
        return 'Ping'             
    
     
api.add_resource(APIResource,                   '/')
api.add_resource(APIVersionResource,            '/version')
api.add_resource(CheckAuthTokenResource,        '/auth-check')

merchant_api.add_resource(AuthenticateAPIResource,       '/auth')    
    
        
        
        
        


'''
Created on 1 Jul 2021

@author: jacklok
'''
from functools import wraps
from flask import request, session, abort
from trexlib.utils.string_util import is_not_empty
from trexlib.utils.crypto_util import decrypt_json
import logging
from datetime import datetime

logger = logging.getLogger('decorator')


def auth_token_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        auth_token  = request.headers.get('x-auth-token')
        acct_id     = request.headers.get('x-acct-id')
            
        logger.debug('acct_id=%s', acct_id)
        logger.debug('auth_token=%s', auth_token)
        
        if is_not_empty(auth_token):
            try:
                auth_details_json = decrypt_json(auth_token)
            except:
                return ("Authenticated token is not valid", 401)
            
            logger.debug('auth_details_json=%s', auth_details_json)
            
            if auth_details_json:
                expiry_datetime     = auth_details_json.get('expiry_datetime')
                acct_id_from_token  = auth_details_json.get('acct_id')
                
                logger.debug('acct_id from decrypted token=%s', acct_id_from_token)
                logger.debug('expiry_datetime from decrypted token=%s', expiry_datetime)
                
                if is_not_empty(expiry_datetime) and is_not_empty(acct_id_from_token) and acct_id==acct_id_from_token:
                    expiry_datetime = datetime.strptime(expiry_datetime, '%d-%m-%Y %H:%M:%S')
                    logger.debug('expiry_datetime=%s', expiry_datetime)
                    
                    now             = datetime.now()
                    if now < expiry_datetime: 
                        logger.debug('auth token is still valid')
                        return f(*args, **kwargs)
                    else:
                        logger.debug('auth token is not logger valid')
                        
                        return ("Authenticated token is expired", 401)
                else:
                    return ("Authenticated token is invalid", 401)
        
        return ("Authenticated token is required", 401)

    return decorated_function

def user_auth_token_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        user_auth_token     = request.headers.get('x-auth-token')
        user_reference_code = request.headers.get('x-reference-code')
            
        logger.debug('user_auth_token=%s', user_auth_token)
        
        if is_not_empty(user_auth_token):
            try:
                auth_details_json = decrypt_json(user_auth_token)
            except:
                return ("Authenticated token is not valid", 401)
            
             
            
            logger.debug('auth_details_json=%s', auth_details_json)
            
            if auth_details_json:
                expiry_datetime                 = auth_details_json.get('expiry_datetime')
                user_reference_code_from_token  = auth_details_json.get('reference_code')
                
                if is_not_empty(expiry_datetime) and is_not_empty(user_reference_code_from_token) and user_reference_code_from_token == user_reference_code:
                    expiry_datetime = datetime.strptime(expiry_datetime, '%d-%m-%Y %H:%M:%S')
                    logger.debug('expiry_datetime=%s', expiry_datetime)
                    
                    now             = datetime.now()
                    if now < expiry_datetime: 
                        logger.debug('auth token is still valid')
                        return f(*args, **kwargs)
                    else:
                        logger.debug('auth token is not logger valid')
                        
                        return ("Authenticated token is expired", 401)
            else:
                return ("Authenticated token is invalid", 401)    
        
        return ("Authenticated token is required", 401)

    return decorated_function

def outlet_key_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        outlet_key = request.headers.get('x-outlet-key')
            
        logger.debug('outlet_key=%s', outlet_key)
        
        if is_not_empty(outlet_key):
            logger.debug('Going to execute')
            return f(*args, **kwargs)
            
        
        return ("Outlet Key is required", 401)

    return decorated_function



from flask import Blueprint, request
import logging
from trexlib.utils.log_util import get_tracelog
from trexmodel.utils.model.model_util import create_db_client
from datetime import datetime
from trexlib.utils.string_util import is_not_empty
from trexmodel.models.datastore.user_models import User
from trexadmin.libs.http import create_rest_message
from trexadmin.libs.http import StatusCode
from werkzeug.datastructures import ImmutableMultiDict
from trexapi.forms.user_api_forms import UserRegistrationForm
from trexapi.conf import APPLICATION_NAME, APPLICATION_BASE_URL
from trexmail.email_helper import trigger_send_email
from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet
from trexmodel.models.datastore.customer_models import Customer
from trexmodel.models.datastore.reward_models import CustomerEntitledVoucher,\
    CustomerEntitledTierRewardSummary, CustomerPointReward
from trexapi.decorators.api_decorators import user_auth_token_required
from flask.json import jsonify
from trexadmin.libs.decorators import elapsed_time_trace

user_reward_api_bp = Blueprint('customer_reward_api_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1/user-reward')

logger = logging.getLogger('debug')


@user_reward_api_bp.route('/ping', methods=['GET'])
def ping():
    return create_rest_message('OK', status_code=StatusCode.OK)

@user_reward_api_bp.route('/customer/<customer_key>/summary', methods=['GET'])
@user_auth_token_required
@elapsed_time_trace(trace_key="read_customer_reward_summary")
def read_user_reward_summary(customer_key):
    reference_code  = request.headers.get('x-reference-code')
    logger.debug('reference_code=%s', reference_code)
    #vouchers_list   = []
    tier_rewards    = []
    
    db_client = create_db_client(caller_info="read_user_reward_summary")
    with db_client.context():
        customer = Customer.fetch(customer_key)
        if customer:
            customer_tier_reward_summary    = CustomerEntitledTierRewardSummary.list_tier_reward_summary_by_customer(customer)
            CustomerPointReward.list_by_customer(customer)
            '''
            if customer_vouchers_list:
                for v in customer_vouchers_list:
                    vouchers_list.append(v.to_dict())
            '''        
            if customer_tier_reward_summary:
                for v in customer_tier_reward_summary:
                    tier_rewards.append(v.to_dict())
    
    
    
    result = {
            'reference_code'    : reference_code,
            #'vouchers'          : vouchers_list,
            'tier_rewards'      : tier_rewards,
            'reward_summary'    : customer.reward_summary,
            'prepaid_summary'   : customer.prepaid_summary,
            'voucher_summary'   : customer.entitled_voucher_summary,
            }
    
    return jsonify(result)

    